/*
 * Licensed Materials - Property of IBM
 * (C) Copyright IBM Corp. 2010, 2012 All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/dom-style",
	"dijit/form/TextBox",
	"dijit/form/ValidationTextBox",
	"idx/widget/HoverHelpTooltip",
	"./_CssStateMixin",
	"./_CompositeMixin",
	"dojo/text!./templates/TextBox.html"
], function(declare, lang, domStyle, TextBox, ValidationTextBox, HoverHelpTooltip, _CssStateMixin, _CompositeMixin, template){
	var iForm = lang.getObject("idx.oneui.form", true); // for backward compatibility with IDX 1.2
	
    /**
	 * @name idx.form.TextBox
	 * @class One UI version.
	 */
	 
	return iForm.TextBox = declare("idx.form.TextBox", [ValidationTextBox, _CompositeMixin, _CssStateMixin], {
		/**@lends idx.form.TextBox*/
		
		// summary:
		//		Base class for textbox widgets with the ability to validate content of various types and provide user feedback.
		// tags:
		//		protected

		// instantValidate: Boolean
		//		Fire validation when widget get input by set true, 
		//		fire validation when widget get blur by set false
		instantValidate: false,
		templateString: template,
		baseClass: "idxTextBoxWrap",
		oneuiBaseClass: "dijitTextBox dijitValidationTextBox",

// Changed this from "postCreate" to "buildRendering" since displayMessage() is called
// by the base class prior to postCreate() and "this.messageTooltip" needs to exist
		buildRendering: function(){
			this.inherited(arguments);
			this.messageTooltip = new HoverHelpTooltip({
				connectId: [this.iconNode],
				label: this.message,
				position: this.tooltipPosition,
				forceFocus: false
			});
		},
		
		postCreate: function() {
			this.inherited(arguments);
			if(this.instantValidate){
				// changed this so the "_onFocus" event first checks if "_hasBeenBlurred" is set
				// if not then no validation is performed to give the user a chance to enter the
				// correct value the first time
				this.connect(this, "_onFocus", function(){
					if (this._hasBeenBlurred && (!this._refocusing)) {
						this.validate(this.focused);
					}
				});
				
				// For instant validation we also need to respond to input and revalidate as input
				// is provided. 
				this.connect(this, "_onInput", function(){
					this.validate(this.focused);
				});
			}
// Default validation should work like a normal ValidationTextBox
// None of this code seems to work properly when tested with LoginFrame
//				else{
//				this.connect(this, "_onBlur", function(){
//					this.validate(this.focused);
//				});
//				this.connect(this, "_onFocus", function(){
//					this._set("state", "");
//					if(this.message == ""){return;}
//					this.displayMessage(this.message);
//					this.message = "";
//				});
//				this.connect(this, "_onInput", function(){
//					this.messageTooltip.close();
//				});
//			}
		},
		
		/**
		 * Provides a method to return focus to the widget without triggering
		 * revalidation.  This is typically called when the validation tooltip
		 * is closed.
		 */
		refocus: function() {
			this._refocusing = true;
			this.focus();
			this._refocusing = false;
		},
		
		/**
		* Overridable method to display validation errors/hints
		*/
		displayMessage: function(/*String*/ message, /*Boolean*/ force){
			this.messageTooltip.set("label", message);
			if(message && this.focused || force ){
				var node = domStyle.get(this.iconNode, "visibility") == "hidden" ? this.oneuiBaseNode : this.iconNode;
				this.messageTooltip.open(node);
			}else{
				this.messageTooltip.close();
			}
		},
		
// Cannot forego the built-in _setValueAttr from ValidationTextBox or else we don't get good
// validation behavior
//
//		/**
//		* use set("value", val) to set the value of the Textbox
//		*/
//		_setValueAttr: function(){
//			TextBox.prototype._setValueAttr.apply(this, arguments);
//		}
	});
});
